import numpy as np
import sys
import time as clock_time

# Path to the place where the tool is installed
tool_path="/gpfs/projects/bsc32/cs_collaboration/BestIterationCutter/"
# Path to the cutter template
template_path = tool_path + "/templates/cutter_template.xml"

# Read trace name
try:
   trace = sys.argv[1]
except IndexError:
   print("Necessary argument: trace with functions")
   exit(1)

print("Starting the magic")
start = clock_time.time()

# Read trace and load function calls
process, time, value = np.genfromtxt(trace, dtype='int', unpack='True')

print("for opening the trace: ")
end = clock_time.time()
print(end - start)

start = clock_time.time()

# Find set of different routines
func_id, call_counts = np.unique(value, return_counts=True)
print("for Find set of different routines: ", func_id.shape[0])
end = clock_time.time()
print(end - start)

# Find number of time-steps multiplied for the number of cores, idle and not
start_4ts_count = clock_time.time()

counts = np.bincount(call_counts)
time_steps =  np.argmax(counts)

end_4ts_count  = clock_time.time()

start = clock_time.time()

# Find functions that are called once per time-step
functions_called_once_per_step = func_id[call_counts == time_steps]

print("for finding functions that are called once per time-step: ", functions_called_once_per_step.shape[0])
end = clock_time.time()
print(end - start)

start = clock_time.time()

# Find which routine is called in first place
first_routine = functions_called_once_per_step[0]
print("for finding  which routine is called in first place: ", first_routine)
end = clock_time.time()
print(end - start)

# Find number of nemo_proc_number
nemo_proc = process[value == first_routine]
nemo_proc = np.unique(nemo_proc)
nemo_proc_number = nemo_proc.shape[0]

time_steps = time[(value == first_routine) & (process == nemo_proc[0])].shape[0]
print("for finding number of time-steps: ", time_steps)
print(end_4ts_count  - start_4ts_count )


start = clock_time.time()

# Find the index of the fastest step
time_step = np.ones([nemo_proc_number , time_steps])

ts_min_index = np.zeros(nemo_proc_number, dtype = 'int')

ts_time = np.zeros([nemo_proc_number, time_steps])
#ts_time.fill(np.nan)
for proc in nemo_proc:
    index = proc - nemo_proc[0]
    ts_time[ index ] = time[ (process == proc)  & (value == first_routine)]

    # compute the duration of each ts
    ts_duration = np.diff(ts_time[ index ])

    #find the index of the minimum ts for each proc
    ts_min_index[index] =  np.argmin(ts_duration)

#evaluate the most common index for best ts
counts = np.bincount(ts_min_index)
best_ts_index =  np.argmax(counts)

print("for finding the index of the fastest step: ", best_ts_index)
end = clock_time.time()
print(end - start)

start = clock_time.time()

# Find the start and the end of the best step
best_ts_start = min(ts_time[:,best_ts_index-1])
best_ts_end = max(ts_time[:,best_ts_index])

print("for finding the start and the end of the best step: ", best_ts_start, best_ts_end)
end = clock_time.time()
print(end - start)

# Read template
with open(template_path, "r") as template:
   text = template.read()

print("start paramedir cutter")
# Create paramedir cutter file
with open("generated_cutter.xml", "w") as output:
   text =   text.replace("%START%","%i" % (best_ts_start - 1000) )
   text =   text.replace("%END%","%i" % (best_ts_end + 1000) )
   output.write(text)
